/**
 * This functions sets the dimensions of the SVG-layer equally to those of the container of the digital edition.
 * This allows for correct positioning of paths between lemmata and readings.
 * @function
 */
const setSvgLayer = function setSvgLayer() {
    let svgLayer = document.getElementsByClassName("svg-layer")[0];
    let edition = document.getElementById("edition");
    
    svgLayer.style.top = edition.offsetTop + "px";
    svgLayer.style.height = edition.offsetHeight + "px";
    svgLayer.style.left = edition.offsetLeft + "px";
    svgLayer.style.width = edition.offsetWidth + "px";
}

/**
 * This function vertically positions a span containing a reading, either at the
 * same height as the corresponding lemma, or at a different height when it would
 * overlap other readings.
 * 
 * @function
 * @param {object} rdg A span containing a reading.
 */
const positionRdg = function positionRdg(rdg) {
    let openRdgs = document.getElementsByClassName("rdg-digital--show");
    let lemmaId = rdg.getAttribute("target-lem").replace("#", "");
    let lemma = document.getElementById(lemmaId);
    let rdgPos = lemma.offsetTop;
    let rdgBottom = rdgPos + rdg.offsetHeight;
    
    for (openedRdg of openRdgs) {
        if (openedRdg != rdg) {
            let openedRdgBottom = openedRdg.offsetTop + openedRdg.offsetHeight;
            rdgPos >= openedRdg.offsetTop && rdgPos < openedRdgBottom + 5 || rdgPos < openedRdg.offsetTop - 5 && rdgBottom > openedRdgBottom + 5
                ? rdgPos = openedRdgBottom + 5 
                : rdgPos;
        }
    };
    rdgBottom = rdgPos + rdg.offsetHeight;
    for (openedRdg of openRdgs) {
        if (openedRdg != rdg) {
            let openedRdgBottom = openedRdg.offsetTop + openedRdg.offsetHeight;
            rdgBottom > openedRdg.offsetTop - 5 && rdgBottom <= openedRdgBottom || rdgPos < openedRdg.offsetTop - 5 && rdgBottom > openedRdgBottom + 5
                ? rdgPos = openedRdgBottom + 5 
                : rdgPos;
        }
    };
    rdg.style.top = rdgPos + "px";
    
};

/**
 * This function draws a SVG-path between a lemma and a corresponding reading
 * and displays it.
 * 
 * @function
 * @param {object} lemma A span containing a lemma.
 * @param {object} rdg A span containing a reading.
 */
const drawPath = function drawPath(lemma, rdg) {
    let digitalText = document.getElementById("digital-text");
    let variants = document.getElementById("variants");
    let rdgId = rdg.id;
    let path = document.querySelector("[target-rdg='#" + rdgId + "']");

    let pathXStart = lemma.offsetWidth >= (digitalText.offsetWidth - 30) ? (30) 
        : (lemma.offsetLeft + lemma.offsetWidth / 2) >= digitalText.offsetWidth ? 60 
        : (lemma.offsetLeft + lemma.offsetWidth / 2);
    let pathYStart = lemma.offsetTop + lemma.offsetHeight -1;
    let pathXEnd = variants.offsetLeft - lemma.offsetParent.offsetLeft + 14;
    let pathYEnd = rdg.offsetTop + rdg.offsetHeight / 2;
    
    let pathMod = pathYStart + 10 < pathYEnd ? "q 3 0 3 3 V " + pathYEnd + " q 0 3 3 3"
        : pathYStart - 10 > pathYEnd ? "q 3 0 3 -3 V " + pathYEnd + " q 0 -3 3 -3"
        : "";

    path.setAttribute("d", "M " + pathXStart + " " + pathYStart + " q 0 3 3 3 H " + digitalText.offsetWidth + pathMod + " H " + pathXEnd);
    path.classList.add("rdg-digital-path--visible");
};

/**
 * This function opens a reading, positions it, highlights the corresponding lemma
 * and draws a SVG-path between the two. This function is called when one of the
 * variant buttons is clicked.
 * 
 * @function
 * @param {object} rdg A span containing a reading.
 */
const openRdg = function openRdg(rdg) {
    let lemmaId = rdg.getAttribute("target-lem").replace("#", "");
    let lemma = document.getElementById(lemmaId);
    let path = document.querySelector(".rdg-digital-path[target-rdg='#" + rdg.id + "']");
    
    lemma.classList.add("open-lem");
    positionRdg(rdg);
    rdg.classList.add("rdg-digital--show");
    drawPath(lemma, rdg);
};

/**
 * This function closes a reading when the reading, its corresponding lemma or
 * the variant button is clicked.
 * 
 * @function
 * @param {object} rdg A span containing a reading.
 */
const closeRdg = function closeRdg(rdg) {
    let lemmaId = rdg.getAttribute("target-lem").replace("#", "");
    let lemma = document.getElementById(lemmaId);
    let path = document.querySelector(".rdg-digital-path[target-rdg='#" + rdg.id + "']");
    
    /** do not remove highlighting from corresponding lemma if there is another
     * open reading corresponding to that lemma.
     */
    document.querySelectorAll(".rdg-digital--show[target-lem='#" + lemmaId + "']").length > 1 ? "" : lemma.classList.remove("open-lem");
    rdg.classList.remove("rdg-digital--show");
    rdg.style.top = "";
    path.classList.remove("rdg-digital-path--visible");
};

/**
 * This function handles the highlighting of a lemma when clicked and calls the 
 * display of any relevant readings, as well as drawing the paths between them.
 * 
 * @function
 * @param {object} lem A span containing a lemma.
 */
const openLemma = function openLemma(lem) {
    let id = lem.id;
    let relevantRdgs = document.querySelectorAll(".rdg-digital[target-lem='#" + id + "']");

    lem.classList.add("open-lem");
    for (rdg of relevantRdgs) {
        positionRdg(rdg);
        rdg.classList.add("rdg-digital--show");
        drawPath(lem, rdg);
    };
};

/**
 * This function closes a lemma and all its corresponding readings.
 * 
 * @function
 * @param {object} lem A span containing a lemma.
 */
const closeLemma = function closeLemma(lem) {
    let id = lem.id;
    let relevantRdgs = document.querySelectorAll(".rdg-digital[target-lem='#" + id + "']");
    
    lem.classList.remove("open-lem");
    for (rdg of relevantRdgs) {
        closeRdg(rdg);
    };
};

/**
 * This is a wrapper function, checking whether a clicked lemma is open or closed.
 * 
 * @function
 */
const toggleLemma = function toggleLemma() {
    this.classList.contains("open-lem") ? closeLemma(this) : openLemma(this);
};

/**
 * This function opens or closes all readings of a particular witness. 
 * If there are any closed readings corresponding to that witness, all remaining
 * closed readings are opened. Otherwise, all readings corresponding to that
 * witness will close.
 * 
 * @function
 */
const toggleAllLemmata = function toggleAllLemmata() {
    let witness = this.dataset.target;
    let openRdgs = document.querySelectorAll(".rdg-digital--show.rdg-digital--" + witness);
    let closedRdgs = document.querySelectorAll(":not(.rdg-digital--show).rdg-digital--" + witness);

    if (closedRdgs.length > 0) {
        for (rdg of closedRdgs) {openRdg(rdg)};
    } else {
        for (rdg of openRdgs) {closeRdg(rdg)};
    }
};

/**
 * This function highlights a lemma on mouseover, its correspoding reading or 
 * readings, as well as the path or paths between them.
 * 
 * @function
 */
const mouseOverLem = function mouseOverLem() {
    let id = this.id;
    let rdg = document.querySelector("[target-lem='#" + id + "']");
    let path = document.querySelector(".rdg-digital-path[target-lem='#" + id + "']");
    
    this.classList.toggle("highlight");
    rdg.classList.toggle("highlight");
    path.classList.toggle("highlight");
};

/**
 * This function highlights a reading on mouseover, as well as its corresponding
 * lemma and the path between the two.
 * 
 * @function
 */
const mouseOverRdg = function mouseOverRdg() {
    let id = this.id;
    let lemId = this.getAttribute("target-lem").replace("#", "");
    let lem = document.getElementById(lemId);
    let path = document.querySelector(".rdg-digital-path[target-rdg='#" + id + "']");
    
    this.classList.toggle("highlight");
    lem.classList.toggle("highlight");
    path.classList.toggle("highlight");
};

/**
 * This function sets the dimensions of the SVG-layer, as well as the position
 * of any opened readings and their corresponding paths, when the screen is
 * resized.
 * 
 * @function
 */
const onScreenResize = function onScreenResize() {
    let openRdgs = document.querySelectorAll(".rdg-digital--show");

    setSvgLayer();
    for (rdg of openRdgs) {
        let lemmaId = rdg.getAttribute("target-lem").replace("#", "");
        let lemma = document.getElementById(lemmaId);
        positionRdg(rdg);
        drawPath(lemma, rdg);
    };
};

/**
 * Get all lemmata, readings and the variants button or buttons.
 */
const lems = document.getElementsByClassName("lemma");
const rdgs = document.getElementsByClassName("rdg-digital");
const variantsButtons = document.getElementsByClassName("btn--variants");

document.addEventListener("DOMContentLoaded", setSvgLayer);
window.addEventListener("resize", onScreenResize);

for (i = 0; i < lems.length; i++) {
    lems[i].addEventListener("click", toggleLemma, false);
    lems[i].addEventListener("mouseenter", mouseOverLem, false);
    lems[i].addEventListener("mouseleave", mouseOverLem, false);
};

for (i = 0; i < rdgs.length; i++) {
    rdgs[i].addEventListener("click", function(){closeRdg(this)}, false);
    rdgs[i].addEventListener("mouseenter", mouseOverRdg, false);
    rdgs[i].addEventListener("mouseleave", mouseOverRdg, false);
};

for (i = 0; i < variantsButtons.length; i++) {
    variantsButtons[i].addEventListener("click", toggleAllLemmata, false);
};